// It turns out that .NET 1.1 doesn't expose low level console access, and
// although .NET 2.0 added this to the Console class, it is pretty buggy
// at least in the beta I'm using (for instance if you press the shift
// key, Console.KeyAvailable returns true even though Console.ReadKey()
// will block). Ah well. Lucky it is so easy to call Win32 routines from
// managed code!

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Threading;



class Conio
{
	// modifier flags
	[Flags()]
	public enum Flags : int
	{
		RightAlt   = 0x01,
		LeftAlt    = 0x02,
		RightCtrl  = 0x04,
		LeftCtrl   = 0x08,
		Shift      = 0x10,
		NumLock    = 0x20,
		ScrollLock = 0x40,
		CapsLock   = 0x80,
		Enhanced   = 0x100,

		Alt  = LeftAlt  | RightAlt,
		Ctrl = LeftCtrl | RightCtrl,
	}



	// a keyboard input event
	[StructLayout(LayoutKind.Sequential)]
	public struct Key
	{
		public int   Keydown;
		public short Repeat;
		public short Rawcode;
		public short Scancode;
		public char  Unicode;
		public Flags Flags;


		// cast to the corresponding framework enum type
		public ConsoleKey Keycode
		{
			get { return (ConsoleKey)Rawcode; }
		}
	}



	// input event type tag
	enum EventType : short
	{
		Key   = 0x01,
		Mouse = 0x02,
		Size  = 0x04,
		Menu  = 0x08,
		Focus = 0x10,
	}



	// combined input event
	[StructLayout(LayoutKind.Sequential)]
	struct InputEvent
	{
		public EventType Type;
		public Key Key;
	}



	// a screen coordinate
	[StructLayout(LayoutKind.Sequential)]
	struct Coord
	{
		public short X;
		public short Y;
	}



	// is standard input redirected?
	public static bool InputRedirected()
	{
		int count;

		return !PeekConsoleInput(InputHandle, new InputEvent[1], 1, out count);
	}



	// access to the parent console window
	public class ConsoleWindow : IWin32Window
	{
		public IntPtr Handle
		{
			get { return GetConsoleWindow(); }
		}
	}



	// read a single character
	public static Key Read()
	{
		InputEvent[] events = new InputEvent[1];
		int count;

		for (;;) {
			// pump GUI events while waiting for a keypress
			if (Application.OpenForms.Count > 0) {
				do {
					Application.DoEvents();
					Thread.Sleep(1);
					PeekConsoleInput(InputHandle, events, 1, out count);
				} while (count == 0);
			}

			// read input
			ReadConsoleInput(InputHandle, events, 1, out count);

			if (count == 1) {
				InputEvent e = events[0];

				if ((e.Type == EventType.Key) && (e.Key.Keydown != 0))
					return e.Key;
			}
		}
	}



	// text output function
	public static void Write(int x, int y, string s)
	{
		int discard;

		Coord coord;

		coord.X = (short)x;
		coord.Y = (short)y;

		WriteConsoleOutputCharacter(OutputHandle, s, s.Length, coord, out discard);
	}



	// magic console filehandles
	static IntPtr InputHandle  = GetStdHandle(-10);
	static IntPtr OutputHandle = GetStdHandle(-11);



	// Windows API imports
	[DllImport("kernel32")] static extern IntPtr GetStdHandle(int handle);
	[DllImport("kernel32")] static extern IntPtr GetConsoleWindow();
	[DllImport("kernel32")] static extern bool PeekConsoleInput(IntPtr handle, [Out] InputEvent[] buffer, int length, out int count);
	[DllImport("kernel32")] static extern bool ReadConsoleInput(IntPtr handle, [Out] InputEvent[] buffer, int length, out int count);
	[DllImport("kernel32")] static extern bool WriteConsoleOutputCharacter(IntPtr handle, string chars, int length, Coord pos, out int count);
}

