// All Your Base Are Belong To Us.

using System;
using System.Text;
using System.Globalization;
using System.Collections.Generic;



class Format
{
	// current number base(s)
	static int[] mBases = { 10 };



	// expose the current base settings to the outside world
	public static int[] Bases
	{
		get {
			return mBases;
		}

		set {
			foreach (int b in value) {
				if ((b < 2) || (b > mDigits.Length))
					throw new UsageException(string.Format("Base {0} is not supported. Valid range is 2 to {1}.", b, mDigits.Length));
			}

			mBases = value;
		}
	}



	// pretty print a real number using the current base settings
	public static string Pretty(double val)
	{
		List<string> s = new List<string>();

		foreach (int b in mBases)
			s.Add(ToBase(b, val));

		return string.Join("\t", s.ToArray());
	}



	// pretty print a complex number using the current base settings
	public static string Pretty(Complex val)
	{
		List<string> s = new List<string>();

		foreach (int b in mBases)
			s.Add(val.ToString(delegate(double v) { return ToBase(b, v); } ));

		return string.Join("\t", s.ToArray());
	}



	// convert real to string, in an arbitrary base
	static string ToBase(int b, double val)
	{
		if (b == 10)
			return val.ToString(CultureInfo.InvariantCulture);
		else
			return ToBase(b, (uint)Math.Round(val));
	}



	// convert integer to string, in an arbitrary base
	static string ToBase(int b, uint val)
	{
		string prefix = (b == 16) ? "0x" : "";

		StringBuilder s = new StringBuilder(prefix);

		long p = b;
		int i = 1;

		while (p <= ShowBits(b, val)) {
			p *= b;
			i++;
		}

		while (i-- > 0) {
			p /= b;
			s.Append(mDigits[(int)((val / p) % b)]);
		}

		return s.ToString();
	}



	// how many bits do we need to display this value?
	static uint ShowBits(int b, uint val)
	{
		if ((b == 2) || (b == 16)) {
			if (val <= 0xFF)
				return 0xFF;
			else if (val <= 0xFFFF)
				return 0xFFFF;
			else
				return 0xFFFFFFFF;
		}
		else
			return val;
	}



	// list of digit characters	
	const string mDigits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
}
