// Assorted helper functions.

using System;
using System.Collections.Generic;



class Util
{
	// functional programming reduce operator (aka inject in Smalltalk/Ruby)
	public delegate T ReduceHandler<T, I> (T t, I i);

	public static T Reduce<T, I> (IEnumerable<I> input, T inject, ReduceHandler<T, I> reducer)
	{
		foreach (I i in input)
			inject = reducer(inject, i);

		return inject;
	}



	// functional programming zip operator (aka interleave)
	public delegate T ZipHandler<T, A, B> (A a, B b);

	public static List<T> Zip<T, A, B> (IEnumerable<A> inputa, IEnumerable<B> inputb, ZipHandler<T, A, B> zipper)
	{
		List<T> zipped = new List<T>();

		IEnumerator<A> a = inputa.GetEnumerator();
		IEnumerator<B> b = inputb.GetEnumerator();

		while (a.MoveNext() && b.MoveNext())
			zipped.Add(zipper(a.Current, b.Current));

		return zipped;
	}



	// generic helper for clamping a value to a range
	public static T Clamp<T> (T low, T mid, T high) where T : IComparable<T>
	{
		if (low.CompareTo(mid) > 0)
			return low;

		if (high.CompareTo(mid) < 0)
			return high;

		return mid;
	}



	// generic helper for sorting a collection
	public static T[] Sort<T> (ICollection<T> collection)
	{
		T[] sorted = new T[collection.Count];

		collection.CopyTo(sorted, 0);

		Array.Sort(sorted);

		return sorted;
	}



	// helper for printing a block of wordwrapped help text
	public static void WriteHelp(string title, string[] text)
	{
		Console.WriteLine("\n{0}:\n", title);

		// find the longest string
		int max = Reduce(text, 0, delegate(int i, string s) { return Math.Max(i, s.Length); } );

		// how many can we fit on a line?
		const string prefix = "    ";

		max += prefix.Length;

		int perLine = 60 / max;
		int count = 0;

		// print the text
		foreach (string s in text) {
			if (count == 0)
				Console.Write(prefix);

			Console.Write(s.PadRight(max));

			if (++count >= perLine) {
				Console.Write("\n");
				count = 0;
			}
		}

		// line feed
		if (count > 0)
			Console.Write("\n");

		Console.Write("\n");
	}
}
