// Does work in a background thread, then invokes a UI completion handler.
// Also notifies the UI thread if the calculation is taking a long time.

using System.Threading;
using System.Windows.Forms;



class WorkerThread
{
    // run arbitrary code in a background thread
    public void BackgroundTask(Control widget, MethodInvoker worker, MethodInvoker busy, MethodInvoker done)
    {
        Stop();

        // run the worker, then call the done hook in the context of a UI widget
        mThread = new Thread(
            delegate() {
                worker();

                StopTimer();

                widget.Invoke(done);

                lock (this) {
                    mThread = null;
                }
            }
        );

        // if this is taking too long, call the UI busy hook
        mTimer = new System.Windows.Forms.Timer();

        mTimer.Interval = 100;

        mTimer.Tick += delegate {
            StopTimer();
            busy();
        };

        // go go go!
        mTimer.Start();
        mThread.Start();
    }



    // forcibly kill the current task
    public void Stop()
    {
        StopTimer();

        lock (this) {
            if (mThread != null) {
                mThread.Abort();
                mThread = null;
            }
        }
    }



    // halt the duration timer
    void StopTimer()
    {
        lock (this) {
            if (mTimer != null) {
                mTimer.Stop();
                mTimer = null;
            }
        }
    }



    // worker thread
    Thread mThread;



    // timer for noticing if the task is taking a long time
    System.Windows.Forms.Timer mTimer;
}

